@@ -32,7 +32,7 @@
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
-
+#include <math.h>
 
 #include "callbacks.h"
 
@@ -42,11 +42,18 @@
 #include "note.h"
 #include "serializer.h"
 #include "deserializer.h"
+#include "search_window.h"
+
+#ifndef max
+	#define max( a, b ) ( ((a) > (b)) ? (a) : (b) )
+#endif
+
 
 /* Private. TODO: Move to some public file */
 GtkTextTag* iter_get_depth_tag(GtkTextIter* iter);
 GtkTextTag* buffer_get_depth_tag(GtkTextBuffer *buffer, gint depth);
 
+static
 GtkTextTag* get_depth_tag_at_line(GtkTextBuffer *buffer, gint line_number)
 {
 	GtkTextIter iter;
@@ -374,104 +381,27 @@ on_link_button_clicked				   (GtkAction		*action,
 	note_show_by_title(text);
 }
 
-
-/*
-static void
-menu_position ( GtkMenu *menu ,
-                gint *x,
-                gint *y,
-                gboolean *push_in ,
-                gpointer *thw)
-{
-    g_return_if_fail (thw -> button );
-    * push_in = TRUE;
-    *x = thw ->button -> allocation .x + thw ->button -> allocation . width ;
-    *y = thw ->button -> allocation .y;
-}
-*/
-
-/* TODO: Put in util file, it's used in interface.c too */
-static
-void set_menu_item_label(GtkMenuItem *item, const gchar *text)
-{
-	GList *children = gtk_container_get_children(GTK_CONTAINER(item));
-	
-	if (children == NULL) {
-		g_printerr("ERROR: set_menu_item_label() expects a GtkMenuItem, which already contains a label.\n");
-		return;
-	}
-	
-	gtk_label_set_markup(GTK_LABEL(children->data), text);		
-}
-
 void
 on_notes_button_clicked				   (GtkAction		*action,
 										gpointer		 user_data) {
 	
-	GtkWidget *mainwin = GTK_WIDGET(user_data);
-	GtkWidget *menu;
 	AppData *app_data;
-	GList *notes;
-	GtkWidget *image_small;
-
+	GTimer *timer;
+	gulong micro;
+	
 	app_data = get_app_data();
-	if (app_data->all_notes == NULL) {
+	if (app_data->note_store == NULL) {
 		return;
 	}
 	
-	menu = gtk_menu_new();
-	gtk_menu_attach_to_widget(GTK_MENU(menu), mainwin, NULL);
+	timer = g_timer_new();
 	
+	search_window_open();
 	
-	/* TODO: This might become a perf. problem with many notes */
-	/* Then we should take care of sorting when inserting and updating elements */
-	app_data->all_notes = sort_note_list_by_change_date(app_data->all_notes);
+	g_timer_stop(timer);
+	g_timer_elapsed(timer, &micro);
+	g_printerr("Opening search window: %lu micro seconds \n", micro);
 	
-	notes = app_data->all_notes;
-	while(notes != NULL) {
-		GtkWidget *menu_item;
-		/*GtkWidget *image_small, *image_large;*/
-		Note *note = notes->data;
-		/* TODO: When starting from Eclipse, use local paths, not from /usr/share/ */
-		image_small = gtk_image_new_from_icon_name("conboy", GTK_ICON_SIZE_SMALL_TOOLBAR);	
-		menu_item = gtk_image_menu_item_new_with_label(note->title); /*gtk_menu_item_new_with_label(note->title);*/
-		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item), image_small);
-		
-		/*set_menu_item_label(GTK_MENU_ITEM(menu_item), g_strconcat("<span size=\"large\">", note->title, "</span>", NULL));*/
-		
-		/*
-		menu_item = hildon_thumb_menu_item_new_with_labels(note->title, note->title, "Open Note...");
-		image_small = gtk_image_new_from_icon_name("conboy", GTK_ICON_SIZE_SMALL_TOOLBAR);
-		image_large = gtk_image_new_from_icon_name("conboy", GTK_ICON_SIZE_DIALOG);
-		hildon_thumb_menu_item_set_images(HILDON_THUMB_MENU_ITEM(menu_item), image_small, image_large);
-		*/
-		
-		g_signal_connect(menu_item, "activate",
-		            	 G_CALLBACK(on_notes_menu_item_activated),
-		            	 note);
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_item);
-		
-		if (notes->next != NULL) {
-			gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_separator_menu_item_new());
-		}
-		
-		notes = notes->next;
-	}
-	
-	/*hildon_menu_set_thumb_mode(GTK_MENU(menu), TRUE);*/
-	
-	gtk_widget_show_all(menu);
-
-	gtk_menu_popup(GTK_MENU (menu), NULL, NULL, NULL, NULL, 0, gtk_get_current_event_time());
-	
-}
-
-void
-on_notes_menu_item_activated		(GtkMenuItem *menuitem,
-									 gpointer     user_data)
-{
-	Note *note = (Note*)user_data;
-	note_show(note);
 }
 
 /* TODO: The signal "mark-set" is emitted 4 times when clicking into the text. While selecting
@@ -688,7 +618,8 @@ on_delete_button_clicked			   (GtkAction		*action,
 	
 	app_data = get_app_data();
 	/* TODO: Free note */
-	app_data->all_notes = g_list_remove(app_data->all_notes, note);
+	/*app_data->all_notes = g_list_remove(app_data->all_notes, note);*/
+	note_list_store_remove(app_data->note_store, note);
 }
 
 static
@@ -710,11 +641,11 @@ on_text_buffer_modified_changed			(GtkTextBuffer *buffer,
 	Note *note = (Note*)user_data;
 	
 	if (!gtk_text_buffer_get_modified(buffer)) {
-		g_printerr("Buffer changed from dirty to saved \n");
+		/*g_printerr("Buffer changed from dirty to saved \n");*/
 		return;
 	}
 	
-	g_printerr("Buffer is dirty. Saving in 10 seconds\n");
+	/*g_printerr("Buffer is dirty. Saving in 10 seconds\n");*/
 	
 	/* Save 10 seconds after the buffer got dirty */
 	g_timeout_add(4000, (GSourceFunc)note_save_callback, note);
@@ -869,6 +800,7 @@ gboolean line_needs_bullet(GtkTextBuffer *buffer, gint line_number)
 static gboolean add_new_line(Note *note)
 {
 	GtkTextBuffer *buffer = note->ui->buffer;
+	GtkTextView *view = note->ui->view;
 	GtkTextIter iter;
 	GtkTextTag *depth_tag;
 	gint line;
@@ -894,6 +826,7 @@ static gboolean add_new_line(Note *note)
 			/* Insert newline and bullet */
 			gtk_text_buffer_get_iter_at_mark(buffer, &iter, gtk_text_buffer_get_insert(buffer));
 			gtk_text_buffer_insert(buffer, &iter, "\n", -1);
+			gtk_text_view_scroll_mark_onscreen(view, gtk_text_buffer_get_insert(buffer));
 			line = gtk_text_iter_get_line(&iter);
 			add_bullets(buffer, line, line, depth_tag);
 			
@@ -931,6 +864,7 @@ static gboolean add_new_line(Note *note)
 	
 		/* If line start with a "- " or "- *" */
 		if (line_needs_bullet(buffer, gtk_text_iter_get_line(&iter))) {
+			GSList *tmp;
 			GtkTextIter *end_iter;
 			
 			gtk_text_iter_set_line_offset(&iter, 0);
@@ -951,9 +885,7 @@ static gboolean add_new_line(Note *note)
 			add_bullets(buffer, line, line, buffer_get_depth_tag(buffer, 1));
 			
 			
-			/* Copied from above */
-			
-			GSList *tmp;
+			/* TODO: Copied from above */
 						
 			/* Remove all tags but <list> from active tags */
 			tmp = g_slist_copy(note->active_tags);
@@ -1003,6 +935,204 @@ on_text_view_key_pressed                      (GtkWidget   *widget,
 	
 }
 
+/* TODO: Optimize this. We need a place where all titles are stored, info which is the longest, ... */
+static gint
+get_length_of_longest_title()
+{
+	gint max_length = 0;
+	AppData *app_data = get_app_data();
+	
+	GList *notes = app_data->search_list;
+	while (notes != NULL) {
+		max_length = max(g_utf8_strlen(((Note*)notes->data)->title, -1), max_length);
+		notes = notes->next;
+	}
+	
+	return max_length;
+}
+
+typedef struct {
+	gint   start_offset;
+	gint   end_offset;
+	Note  *note;
+} SearchHit;
+
+static
+GSList* find_titles(gchar *haystack) {
+	/* TODO: The search algorithm needs to be optimized. Probably to use Aho-Corasick */
+	/* ATM On the device (N810) searching takes around 1500 microseconds with ~70 notes. */
+	
+	/* After the change from keeping all notes in a simple GList to using a GtkTreeModel
+	 * search time increased dramatically to around 15000 micro seconds on the PC. Because
+	 * of this I reintroduced a simple list representation which is only for searching.
+	 * Now the time is back to around 150 micro seconds on the PC.
+	 * This simple list will probably be replaced by some Hash-Thing in the future. But it's
+	 * obvious, that we cannot work direktly on the GtkTreeModel :(
+	 */
+	
+	GTimer *search_timer = g_timer_new();
+	
+	Note *note;
+	gchar *found;
+	GSList *result = NULL;
+	AppData *app_data = get_app_data();
+	
+	gchar *u_haystack = g_utf8_casefold(haystack, -1);
+	gulong micro;
+	
+	GList *notes = app_data->search_list;
+	while (notes != NULL) {
+		gchar *u_needle;
+		note = (Note*)notes->data;
+		
+		u_needle = g_utf8_casefold(note->title, -1);
+		
+		found = u_haystack;
+		
+		while ( (found = strstr(found, u_needle)) != NULL ) {
+			SearchHit *hit = g_new0(SearchHit, 1);
+			hit->note = note;
+			hit->start_offset = g_utf8_pointer_to_offset(u_haystack, found);
+			hit->end_offset = hit->start_offset + g_utf8_strlen(u_needle, -1);
+			result = g_slist_prepend(result, hit);
+			found = found + strlen(u_needle);
+		}
+		g_free(u_needle);
+		notes = notes->next;
+	}
+	
+	g_timer_stop(search_timer);
+	
+	g_timer_elapsed(search_timer, &micro);
+	g_printerr("Search took %lu microseconds \n", micro);
+	
+	g_free(u_haystack);
+	
+	return result;
+}
+
+static void
+extend_block(GtkTextIter *start_iter, GtkTextIter *end_iter, gint max_len, GtkTextTag *tag)
+{	
+	/* Set start_iter max_len chars to the left or to the start of the line */
+	if (gtk_text_iter_get_line_offset(start_iter) - max_len > 0) {
+		gtk_text_iter_backward_chars(start_iter, max_len);
+	} else {
+		gtk_text_iter_set_line_offset(start_iter, 0);
+	}
+	
+	/* Set end_iter max_len chars to the right or to the end of the line */
+	if (gtk_text_iter_get_line_offset(end_iter) + max_len < gtk_text_iter_get_chars_in_line(end_iter) ) {
+		gtk_text_iter_forward_chars(end_iter, max_len);
+	} else {
+		gtk_text_iter_forward_to_line_end(end_iter);
+	}
+	
+	/* Expand selection to the left, if there is a link_tag inside */
+	if (gtk_text_iter_has_tag(start_iter, tag)) {
+		gtk_text_iter_backward_to_tag_toggle(start_iter, tag);
+	}
+	
+	/* Expand selection to the right, if there is a link_tag inside */
+	if (gtk_text_iter_has_tag(end_iter, tag)) {
+		gtk_text_iter_forward_to_tag_toggle(end_iter, tag);
+	}
+}
+
+static void
+highlight_titles(Note *note, GtkTextBuffer *buffer, GtkTextIter *start_iter, GtkTextIter *end_iter)
+{
+	GtkTextTag *url_tag  = gtk_text_tag_table_lookup(buffer->tag_table, "link:url");
+	/* For all titles look if they occure in haystack. For all matches apply tag */
+	GSList *hits = find_titles(gtk_text_buffer_get_slice(buffer, start_iter, end_iter, FALSE));
+	while (hits != NULL) {
+		GtkTextIter *hit_start, *hit_end;
+		SearchHit *hit = (SearchHit*)hits->data;
+		
+		hit_start = gtk_text_iter_copy(start_iter);
+		gtk_text_iter_forward_chars(hit_start, hit->start_offset);
+		
+		hit_end = gtk_text_iter_copy(start_iter);
+		gtk_text_iter_forward_chars(hit_end, hit->end_offset);
+		
+		/* Only link agains words or sentencens */
+		if ( (!gtk_text_iter_starts_word(hit_start) && !gtk_text_iter_starts_sentence(hit_start)) || 
+				(!gtk_text_iter_ends_word(hit_end) && !gtk_text_iter_ends_sentence(hit_end))) {
+			hits = hits->next;
+			continue;
+		}
+		
+		/* Don´t link against the note itself */
+		if (hit->note == note) {
+			hits = hits->next;
+			continue;
+		}
+		
+		/* Don't create links inside external links */
+		if (gtk_text_iter_has_tag(hit_start, url_tag)) {
+			hits = hits->next;
+			continue;
+		}
+
+		/* Apply the tag */
+		gtk_text_buffer_apply_tag_by_name(buffer, "link:internal", hit_start, hit_end);
+		
+		hits = hits->next;
+	}
+	
+	g_slist_free(hits);
+}
+
+static void
+auto_highlight_links(GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *input, Note *note)
+{
+	GtkTextIter *start_iter, *end_iter;
+	GtkTextTag *link_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
+	int max_len = get_length_of_longest_title();
+	
+	start_iter = gtk_text_iter_copy(iter);
+	end_iter = gtk_text_iter_copy(iter);
+	
+	/* Move start iter back to the position before the insert */
+	gtk_text_iter_backward_chars(start_iter, g_utf8_strlen(input, -1));
+	
+	extend_block(start_iter, end_iter, max_len, link_tag);
+	
+	/* Remove existing link tag */
+	gtk_text_buffer_remove_tag(buffer, link_tag, start_iter, end_iter);
+	
+	/* Add links */
+	highlight_titles(note, buffer, start_iter, end_iter);
+}
+
+static void
+apply_active_tags(GtkTextBuffer *buffer, GtkTextIter *iter, const gchar *input, Note *note)
+{
+	GtkTextIter *start_iter;
+	GSList *active_tags = note->active_tags;
+	
+	/* Only apply active tags on typed text, not on pasted text */
+	if (g_utf8_strlen(input, -1) > 1) {
+		return;
+	}
+	
+	/* The first line is always the title. Don't apply tags there */
+	if (gtk_text_iter_get_line(iter) == 0) {
+		return;
+	}
+		
+	/* First remove all tags, then apply all active tags */
+	start_iter = gtk_text_iter_copy(iter);
+	gtk_text_iter_backward_chars(start_iter, g_utf8_strlen(input, -1));
+	gtk_text_buffer_remove_all_tags(buffer, start_iter, iter);
+	
+	while (active_tags != NULL && active_tags->data != NULL) {
+		gtk_text_buffer_apply_tag(buffer, active_tags->data, start_iter, iter);
+		active_tags = active_tags->next;
+	}
+
+}
+
 void
 on_text_buffer_insert_text					(GtkTextBuffer *buffer,
 											 GtkTextIter   *iter,
@@ -1010,27 +1140,39 @@ on_text_buffer_insert_text					(GtkTextBuffer *buffer,
 											 gint			len,
 											 gpointer		user_data)
 {
-	/* TODO: Finding titles to create links */
-	GSList *active_tags;
-	GtkTextIter *start_iter;
+	Note *note = (Note*)user_data;
+	GTimer *timer;
+	gulong micro;
 	
-	/* The first line is always the title. Don't apply tags there */
-	if (gtk_text_iter_get_line(iter) == 0) {
-		return;
-	}
+	apply_active_tags(buffer, iter, text, note);
 	
-	active_tags = ((Note*)user_data)->active_tags;
-	start_iter = gtk_text_iter_copy(iter);
-	gtk_text_iter_backward_char(start_iter);
+	timer = g_timer_new();
 	
-	gtk_text_buffer_remove_all_tags(buffer, start_iter, iter);
+	auto_highlight_links(buffer, iter, text, note);
 	
-	while (active_tags != NULL && active_tags->data != NULL) {
-		gtk_text_buffer_apply_tag(buffer, active_tags->data, start_iter, iter);
-		active_tags = active_tags->next;
-	}
-	return;
+	g_timer_stop(timer);
+	g_timer_elapsed(timer, &micro);
+	g_printerr("Complete insert: %lu ms \n", micro);
+}
+
+void
+on_text_buffer_delete_range					(GtkTextBuffer *buffer,
+											 GtkTextIter   *start_iter,
+											 GtkTextIter   *end_iter,
+											 gpointer		user_data)
+{
+	Note *note = (Note*)user_data;
+	GtkTextTag *link_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
+	gint max_len = get_length_of_longest_title();
 	
+	/* This handler runs after the default handler, so we can modify the iters as we like */
+	extend_block(start_iter, end_iter, max_len, link_tag);
+	
+	/* Remove link tags */
+	gtk_text_buffer_remove_tag(buffer, link_tag, start_iter, end_iter);
+	
+	/* Add link tags */
+	highlight_titles(note, buffer, start_iter, end_iter);
 }
 
 /*
@@ -1060,6 +1202,7 @@ GtkTextTag* buffer_get_depth_tag(GtkTextBuffer *buffer, gint depth)
 	return tag;
 }
 
+static
 gboolean tag_is_depth_tag(GtkTextTag *tag)
 {
 	if (tag == NULL) {
@@ -1068,6 +1211,7 @@ gboolean tag_is_depth_tag(GtkTextTag *tag)
 	return (strncmp(tag->name, "depth", 5) == 0);
 }
 
+
 gint tag_get_depth(GtkTextTag *tag)
 {
 	if (tag_is_depth_tag(tag)) {
@@ -1091,6 +1235,7 @@ GtkTextTag* iter_get_depth_tag(GtkTextIter* iter)
 	return NULL;
 }
 
+static
 void increase_indent(GtkTextBuffer *buffer, gint start_line, gint end_line)
 {
 	GtkTextIter start_iter, end_iter;
@@ -1124,6 +1269,7 @@ void increase_indent(GtkTextBuffer *buffer, gint start_line, gint end_line)
 }
 
 /* TODO: This is almost 100% copy&paste from increase_indent */
+static
 void decrease_indent(GtkTextBuffer *buffer, gint start_line, gint end_line)
 {
 	GtkTextIter start_iter, end_iter;
