@@ -134,6 +134,12 @@ on_text_buffer_insert_text					(GtkTextBuffer *buffer,
 											 gint			len,
 											 gpointer		user_data);
 
+void
+on_text_buffer_delete_range					(GtkTextBuffer *buffer,
+											 GtkTextIter   *start_iter,
+											 GtkTextIter   *end_iter,
+											 gpointer		user_data);
+
 void
 on_inc_indent_button_clicked			   (GtkAction		*action,
 											gpointer		 user_data);
